require('sketchup')

module GKWare
  #
end

module GKWare::MacTest
  def self.plugin()
    @plugin
  end

  class TestMac
    include GKWare::MacTest

    def initialize()
      lines = []

      windows = (RUBY_PLATFORM =~ /darwin/i) == nil
      cmd = windows ? 'ipconfig /all' : 'ifconfig'

      if (! windows || (RUBY_VERSION[0..2] == '1.8'))
        lines = %x[#{cmd}].split("\n")
      else
        temp_path = File.expand_path(ENV['TMPDIR'] || ENV['TMP'] || ENV['TEMP'])
        id = "#{Time.now}".hash.abs
        tempfile = File.join(temp_path, "#{id}.txt")
        
        system("#{cmd} 1> #{tempfile}")

        begin
          t1 = Time.new.to_i
          file = File.open(tempfile, 'r', :encoding => Encoding::find("filesystem"))
          t2 = file.ctime.to_i
          lines = file.read.split("\n")
          file.close

          File.unlink(tempfile)

          raise "Error checking computer" if ((t2 - t1).abs > 1)
        rescue StandardError => e
          UI.messagebox(e)
          return ""
        end
      end

      mac = process_lines(lines)
      puts mac
    end
    
    def process_lines(lines)
      macs = Hash.new(0)
      regex = Regexp.compile('([0-9A-F]{2}[:]){5}([0-9A-F]{2})$')

      state = 0
      tmp   = ""

      lines.each { |line|
        next if (line.nil? || line.empty?)

        # if first character is not a space then we are at beginning of block
        state = 1 if (line[0,1] != " ")

        # for consistency we need to convert - to : and make upper case and strip spaces of right side
        test = line.gsub(/-/, ':').upcase().rstrip()

        # returns last 17 characters of mac address - might be a DHCPv6
        tmp = regex.match(test).to_s
        found = (tmp != nil && tmp != "")

        # looking for en0 or Ethernet - this effectively filters out other devices
        state = 2 if (state == 1 && (line.include?("Ethernet") || line.include?("en0")))
        macs[tmp] += 1 if (found && state == 2)
      }

      # Use the Mac that appears the most times
      # Where there is a tie the first one is used
      mac = ""
      best = 0
      macs.each { |key, value| 
        if (best < value)
          best = value
          mac = key
        end
      }
      
      mac
    end
  end

  if (not file_loaded?('gkware_testmac'))
    # add menu items
    plugins_menu = UI.menu('Plugins')
    plugins_menu.add_item('Mac Address') { TestMac.new() }

    file_loaded('gkware_testmac')
  end
  # ----------------------------
end
